package transactionFeature;

import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.Environment;
import com.sleepycat.je.RunRecoveryException;

privileged public aspect EnvironmentExtension {
	
	
	before (Environment env): execution(com.sleepycat.je.Environment.new(..)) && this(env){
		env.referringDbTxns = Collections.synchronizedSet(new HashSet());
	}
	
	/**
	 * Let the Environment remember what's opened against it.
	 */
	void com.sleepycat.je.Environment.addReferringHandle(Transaction txn) {
		referringDbTxns.add(txn);
	}
	
	/**
	 * The referring Transaction has been closed.
	 */
	void com.sleepycat.je.Environment.removeReferringHandle(Transaction txn) {
		referringDbTxns.remove(txn);
	}
	private Set com.sleepycat.je.Environment.referringDbTxns;

	
	
	after (Environment env,StringBuffer errors) throws RunRecoveryException, DatabaseException : 
		call (void Environment.hook_closeTransactions(StringBuffer)) && args(errors) && this(env)
	{
		if (env.referringDbTxns != null) {
			int nTxns = env.referringDbTxns.size();
			if (nTxns != 0) {
				Iterator iter = env.referringDbTxns.iterator();
				errors.append("There ");
				if (nTxns == 1) {
					errors
							.append("is 1 existing transaction opened against");
					errors.append(" the Environment.\n");
				} else {
					errors.append("are ");
					errors.append(nTxns);
					errors.append(" existing transactions opened against");
					errors.append(" the Environment.\n");
				}
				errors.append("Aborting open transactions ...\n");

				while (iter.hasNext()) {
					Transaction txn = (Transaction) iter.next();
					try {
						txn.abort();
					} catch (RunRecoveryException e) {
						throw e;
					} catch (DatabaseException DBE) {
						errors.append("\nWhile aborting transaction ");
						errors.append(txn.getId());
						errors.append(" encountered exception: ");
						errors.append(DBE).append("\n");
					}
				}
			}
		}
	}
	
}
