package transactionFeature;

import com.sleepycat.je.Cursor;
import com.sleepycat.je.CursorConfig;
import com.sleepycat.je.Database;
import com.sleepycat.je.DatabaseConfig;
import com.sleepycat.je.DatabaseEntry;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.Environment;
import com.sleepycat.je.LockMode;
import com.sleepycat.je.OperationContext;
import com.sleepycat.je.OperationStatus;
import com.sleepycat.je.SecondaryConfig;
import com.sleepycat.je.SecondaryDatabase;
import com.sleepycat.je.Sequence;
import com.sleepycat.je.SequenceConfig;

public aspect InterfaceFunctions {

	public synchronized Database Environment.openDatabase(Transaction txn,
			String databaseName, DatabaseConfig dbConfig)
			throws DatabaseException {
		return openDatabase(new OperationContext(txn), databaseName, dbConfig);
	}

	public synchronized SecondaryDatabase Environment.openSecondaryDatabase(
			Transaction txn, String databaseName, Database primaryDatabase,
			SecondaryConfig dbConfig) throws DatabaseException {
		return openSecondaryDatabase(new OperationContext(txn), databaseName, primaryDatabase, dbConfig);
	}
//TODO make this a derivative
//	public void Environment.removeDatabase(Transaction txn, String databaseName)
//			throws DatabaseException {
//		removeDatabase(databaseName);
//	}

	public void Environment.renameDatabase(Transaction txn,
			String databaseName, String newName) throws DatabaseException {
		renameDatabase(new OperationContext(txn), databaseName, newName);
	}

//	TODO make this a derivative
//	public long Environment.truncateDatabase(Transaction txn,
//			String databaseName, boolean returnCount) throws DatabaseException {
//		return truncateDatabase(databaseName, returnCount);
//	}

	/**
	 * database
	 */

	public Sequence Database.openSequence(Transaction txn, DatabaseEntry key,
			SequenceConfig config) throws DatabaseException {
		return openSequence(new OperationContext(txn), key, config);
	}

	public void Database.removeSequence(Transaction txn, DatabaseEntry key)
			throws DatabaseException {
		removeSequence(new OperationContext(txn), key);
	}

	public synchronized Cursor Database.openCursor(Transaction txn,
			CursorConfig cursorConfig) throws DatabaseException {
		return openCursor(new OperationContext(txn), cursorConfig);
	}

	public OperationStatus Database.delete(Transaction txn, DatabaseEntry key)
			throws DatabaseException {
		return delete(new OperationContext(txn), key);
	}

	public OperationStatus Database.get(Transaction txn, DatabaseEntry key,
			DatabaseEntry data, LockMode lockMode) throws DatabaseException {
		return get(new OperationContext(txn), key, data, lockMode);
	}

	public OperationStatus Database.getSearchBoth(Transaction txn,
			DatabaseEntry key, DatabaseEntry data, LockMode lockMode)
			throws DatabaseException {
		return getSearchBoth(new OperationContext(txn), key, data, lockMode);
	}

	public OperationStatus Database.put(Transaction txn, DatabaseEntry key,
			DatabaseEntry data) throws DatabaseException {
		return put(new OperationContext(txn), key, data);
	}

	public OperationStatus Database.putNoOverwrite(Transaction txn,
			DatabaseEntry key, DatabaseEntry data) throws DatabaseException {
		return putNoOverwrite(new OperationContext(txn), key, data);
	}

	public OperationStatus Database.putNoDupData(Transaction txn,
			DatabaseEntry key, DatabaseEntry data) throws DatabaseException {
		return putNoDupData(new OperationContext(txn), key, data);
	}


	
	/**
	 * secondarydatabase
	 */

	public synchronized Cursor SecondaryDatabase.openSecondaryCursor(Transaction txn,
			CursorConfig cursorConfig) throws DatabaseException {
		return openCursor(new OperationContext(txn), cursorConfig);
	}

	public OperationStatus SecondaryDatabase.get(Transaction txn, DatabaseEntry key,
			DatabaseEntry pKey, DatabaseEntry data, LockMode lockMode) throws DatabaseException {
		return get(new OperationContext(txn), key, pKey, data, lockMode);
	}
	
	public OperationStatus SecondaryDatabase.getSearchBoth(Transaction txn,
			DatabaseEntry key, DatabaseEntry pKey, DatabaseEntry data, LockMode lockMode)
			throws DatabaseException {
		return getSearchBoth(new OperationContext(txn), key, pKey, data, lockMode);
	}
	

	

	

}
