package transactionFeature;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.dbi.DatabaseId;
import com.sleepycat.je.dbi.DatabaseImpl;
import com.sleepycat.je.dbi.EnvironmentImpl;
import com.sleepycat.je.log.LogEntryType;
import com.sleepycat.je.tree.LN;
import com.sleepycat.je.txn.Locker;
import com.sleepycat.je.txn.Txn;
import com.sleepycat.je.txn.WriteLockInfo;

public aspect LNRef {

	pointcut deleteCall(Locker locker) : execution(long LN.delete(DatabaseImpl , byte[] , byte[] ,
			long , Locker)) && args(..,locker);

	pointcut logCall(Locker locker) : execution(long LN.log(EnvironmentImpl , DatabaseId , byte[] ,
			long , Locker,boolean)) && args(..,locker,boolean);

	LogEntryType around(Locker locker):
		call(LogEntryType LN.hook_getEntryType(LogEntryType)) && cflow(deleteCall(locker))
	{
		if (locker.isTransactional())
			return LogEntryType.LOG_DEL_DUPLN_TRANSACTIONAL;
		else
			return proceed(locker);
	}

	long around(Locker locker, LN ln) throws DatabaseException:
		execution(long LN.hook_getLogAbortLsn(long)) && target(ln) && 
		cflow(deleteCall(locker) || logCall(locker))
	{
		if (locker.isTransactional()) {
			WriteLockInfo info = locker.getWriteLockInfo(ln.getNodeId());
			return info.getAbortLsn();
		} else
			return proceed(locker, ln);
	}

	boolean around(Locker locker, LN ln) throws DatabaseException:
		execution(boolean LN.hook_getLogAbortKnownDeleted(boolean)) && target(ln) && 
		cflow(deleteCall(locker) ||logCall(locker))
	{
		if (locker.isTransactional()) {
			WriteLockInfo info = locker.getWriteLockInfo(ln.getNodeId());
			return info.getAbortKnownDeleted();
		} else
			return proceed(locker, ln);
	}

	Txn around(Locker locker) throws DatabaseException:
		execution(Txn LN.hook_getLogTxn()) &&  
		cflow(deleteCall(locker) ||logCall(locker))
	{
		if (locker.isTransactional()) {
			return locker.getTxnLocker();
		} else
			return proceed(locker);
	}

}
