package transactionFeature;

import com.sleepycat.je.Database;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.DbInternal;
import com.sleepycat.je.Environment;
import com.sleepycat.je.OperationContext;
import com.sleepycat.je.TransactionConfig;
import com.sleepycat.je.dbi.DatabaseImpl;
import com.sleepycat.je.dbi.EnvironmentImpl;
import com.sleepycat.je.txn.AutoTxn;
import com.sleepycat.je.txn.Locker;
import com.sleepycat.je.txn.LockerFactory;

privileged public aspect LockerFactoryRef {

	// // /**
	// // * Get a locker for a writable operation, checking whether the db and
	// // * environment is transactional or not. Must return a non null locker.
	// // */
	// // around()throws DatabaseException :
	// // execution(Locker LockerFactory.getWritableLocker(Environment,
	// OperationContex)) && args(env,cxt){
	// //
	// // return getWritableLocker(env, cxt.getTxn(), cxt.dbIsTransactional,
	// false, null);
	// // }
	//
	// /**
	// * Get a locker for a writable operation, also specifying whether to
	// retain
	// * non-transactional locks when a new locker must be created.
	// *
	// * @param retainNonTxnLocks
	// * is true for DbTree operations, so that the handle lock may be
	// * transferred out of the locker when the operation is complete.
	// */
	//		
	Locker around(Environment env, OperationContext cxt,
			boolean retainNonTxnLocks) throws DatabaseException :
		execution(Locker LockerFactory.getWritableLocker(Environment, OperationContext, boolean)) && 
		args(env,cxt,retainNonTxnLocks){

		Transaction userTxn = cxt.getTxn();
		EnvironmentImpl envImpl = DbInternal.envGetEnvironmentImpl(env);
		boolean envIsTransactional = envImpl.isTransactional();

		if (userTxn == null) {
			Transaction xaLocker = env.getThreadTransaction();
			if (xaLocker != null) {
				return DbInternal.getLocker(xaLocker);
			}
		}

		if (cxt.dbIsTransactional && userTxn == null) {
			TransactionConfig autoCommitConfig = cxt.autoCommitConfig;
			if (autoCommitConfig == null) {
				autoCommitConfig = DbInternal.getDefaultTxnConfig(env);
			}
			return new AutoTxn(envImpl, autoCommitConfig);

		} else if (userTxn == null) {

			return proceed(env, cxt, retainNonTxnLocks);

		} else {

			/*
			 * The user provided a transaction, the environment and the database
			 * had better be opened transactionally.
			 */
			if (!envIsTransactional) {
				throw new DatabaseException(
						"A Transaction cannot be used because the"
								+ " environment was opened"
								+ " non-transactionally");
			}
			if (!cxt.dbIsTransactional) {
				throw new DatabaseException(
						"A Transaction cannot be used because the"
								+ " database was opened"
								+ " non-transactionally");
			}

			/*
			 * Use the locker for the given transaction. For read-comitted, wrap
			 * the given transactional locker in a special locker for that
			 * isolation level. But if retainNonTxnLocks we cannot use
			 * read-committed, since retainNonTxnLocks is used for handle locks
			 * that must be retained across operations.
			 */
			Locker locker = DbInternal.getLocker(userTxn);
			if (locker.isReadCommittedIsolation() && !retainNonTxnLocks) {
				return new ReadCommittedLocker(envImpl, locker);
			} else {
				return locker;
			}
		}
	}

	

	Locker around(Environment env, OperationContext cxt, boolean retainNonTxnLocks,
			boolean readCommittedIsolation) throws DatabaseException :
		execution(Locker LockerFactory.getReadableLocker(Environment, OperationContext,boolean,boolean)) && 
        args(env,cxt,retainNonTxnLocks,readCommittedIsolation){

		Transaction userTxn = cxt.getTxn();
		if (userTxn != null && !cxt.dbIsTransactional) {
			throw new DatabaseException(
					"A Transaction cannot be used because the"
							+ " database was opened" + " non-transactionally");
		}

		Locker locker = null;
		if (userTxn != null) {

			/*
			 * Use the locker for the given transaction. Request read-comitted
			 * if that isolation level is configured for the transaction, or if
			 * true is passed for the parameter (this is the case when
			 * read-committed is configured for the cursor).
			 */
			locker = DbInternal.getLocker(userTxn);
			if (locker.isReadCommittedIsolation()) {
				readCommittedIsolation = true;
			}
		}

		return LockerFactory.getReadableLocker(env, locker, retainNonTxnLocks,
				readCommittedIsolation);
	}

	Locker around(Environment env, Locker locker, boolean retainNonTxnLocks,
			boolean readCommittedIsolation) throws DatabaseException :
		execution(Locker LockerFactory.getReadableLocker(Environment, Locker,boolean,boolean)) && 
        args(env,locker,retainNonTxnLocks,readCommittedIsolation){

		if (locker == null) {
			Transaction xaTxn = env.getThreadTransaction();
			if (xaTxn != null) {
				return DbInternal.getLocker(xaTxn);
			}
		} else{
            /*
             * Use the given locker.  For read-committed, wrap the given
             * transactional locker in a special locker for that isolation
             * level.  But if retainNonTxnLocks we cannot use read-committed,
             * since retainNonTxnLocks is used for handle locks that must be
             * retained across operations.
             */
            if (readCommittedIsolation && !retainNonTxnLocks) {
                EnvironmentImpl envImpl = DbInternal.envGetEnvironmentImpl(env);
                locker = new ReadCommittedLocker(envImpl, locker);
            }
		}
		return proceed(env,locker,retainNonTxnLocks,readCommittedIsolation);
	}
	
	Locker around(Environment env, Database dbHandle, Locker locker, boolean retainNonTxnLocks,
			boolean readCommittedIsolation) throws DatabaseException :
		execution(Locker LockerFactory.getReadableLocker(Environment, Database,Locker,boolean,boolean)) && 
        args(env,dbHandle,locker,retainNonTxnLocks,readCommittedIsolation){
		
		
		DatabaseImpl dbImpl = DbInternal.dbGetDatabaseImpl(dbHandle);
	        if (!dbImpl.isTransactional() && 
	            locker != null &&
	            locker.isTransactional()) {
	            throw new DatabaseException
	                ("A Transaction cannot be used because the" +
	                 " database was opened" +
	                 " non-transactionally");
	        }

	        /* 
	         * Don't reuse a non-transactional locker unless retaining
	         * non-transactional locks was requested.
	         */
	        if (locker != null &&
	            !locker.isTransactional() &&
	            !retainNonTxnLocks) {
	            locker = null;
	        }

	        return proceed
	            (env, dbHandle, locker, retainNonTxnLocks, readCommittedIsolation);
	}

}
