package transactionFeature;

import com.sleepycat.je.Cursor;
import com.sleepycat.je.CursorConfig;
import com.sleepycat.je.Database;
import com.sleepycat.je.DatabaseConfig;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.DbInternal;
import com.sleepycat.je.Environment;
import com.sleepycat.je.OperationContext;
import com.sleepycat.je.Sequence;
import com.sleepycat.je.dbi.EnvironmentImpl;
import com.sleepycat.je.txn.Locker;
import com.sleepycat.je.txn.LockerFactory;

privileged public aspect TransactionLockerConnection {

	pointcut getWritableLocker1(Environment env, OperationContext cxt,
			boolean retainNonTxnLocks): 
		call(Locker LockerFactory.getWritableLocker(Environment,OperationContext,boolean)) && 
		args(env,cxt,retainNonTxnLocks);

	/**
	 * environment (Environment.openDb handled exclusively, for both occurences)
	 */
	before(DatabaseConfig dbConfig, OperationContext cxt):
		execution(void Environment.openDb(OperationContext, Database, String, DatabaseConfig, boolean)) && 
		args(cxt,Database, String,dbConfig,boolean) 
	{

		cxt.dbIsTransactional = dbConfig.getTransactional();
	}

	Locker around(Environment env, OperationContext cxt,
			boolean needWritableLockerForInit) throws DatabaseException : 
		execution(Locker Environment.hook_getWritableLocker(OperationContext,boolean)) && 
		args(cxt,needWritableLockerForInit) && this(env)
	{
		if (needWritableLockerForInit) {
			return LockerFactory.getWritableLocker(env, cxt, true // retainNonTxnLocks
			);
		} else {
			return LockerFactory.getReadableLocker(env, cxt, true, // retainNonTxnLocks
					false); // readCommittedIsolation
		}
	}

	boolean around(Locker locker, DatabaseConfig dbConfig,
			boolean needWritableLockerForInit) : 
		call(boolean Environment.hook_getIsWritableLocker(Locker, DatabaseConfig, boolean)) &&
		args(locker, dbConfig, needWritableLockerForInit)   
	{
		boolean isWritableLocker;
		if (needWritableLockerForInit) {
			isWritableLocker = true;
		} else {
			isWritableLocker = !dbConfig.getTransactional()
					|| locker.isTransactional();
		}
		return isWritableLocker;
	}

	Locker around(Environment env, OperationContext cxt, Database db,
			boolean retainNonTxnLocks) throws DatabaseException :
		call(Locker LockerFactory.getWritableLocker(Environment,OperationContext,boolean)) && 
		args(env,cxt,retainNonTxnLocks) && this(db) {

		cxt.dbIsTransactional = db.isTransactional();
		return LockerFactory.getWritableLocker(env, cxt, retainNonTxnLocks);
	}

	before(Environment env, OperationContext cxt) throws DatabaseException :
		getWritableLocker1(env, cxt, *)	&& 
		within(Environment) && 
		!withincode(void Environment.openDb(OperationContext, Database, String, DatabaseConfig, boolean))
	{
		EnvironmentImpl envImpl = DbInternal.envGetEnvironmentImpl(env);
		cxt.dbIsTransactional = envImpl.isTransactional();
	}

	/**
	 * database
	 */
	before(Database db, OperationContext cxt) throws DatabaseException :
		getWritableLocker1(*, cxt, *)
		&& within(Database) && this(db)
	{
		cxt.dbIsTransactional = db.isTransactional();
	}

	/**
	 * sequence
	 */
	before(Sequence seq, OperationContext cxt) throws DatabaseException :
		getWritableLocker1(*, cxt, *)
		&& within(Sequence) && this(seq)
	{
		cxt.dbIsTransactional = seq.db.isTransactional();
		cxt.autoCommitConfig = seq.autoCommitConfig;
	}

	/**
	 * cursor
	 */
	before(Database dbHandle, OperationContext cxt) throws DatabaseException :
		execution(Cursor.new(Database,OperationContext,CursorConfig)) && args(dbHandle,cxt,*){
		cxt.dbIsTransactional = dbHandle.isTransactional();
	}

	Locker around(Database dbHandle, Locker locker, CursorConfig cursorConfig)
			throws DatabaseException :
		call(Locker Cursor.hook_getReadableLocker(Database,Locker,CursorConfig)) && 
		args(dbHandle,locker,cursorConfig)
	{
		return LockerFactory.getReadableLocker(dbHandle.getEnvironment(),
				dbHandle, locker, false, cursorConfig.getReadCommitted());
	}

}
